/*
 * Csucslista.java
 * a jtk cscslistjnak adatszerkezete
 * by pts@fazekas.hu at Sun Apr 22 19:28:17 CEST 2001
 * tovbbi doksi: a README-ben s e file legkls class-nak fejkommentjben
 *
 * Kincskeres Kisgmbc (C) Early May 2001 by eNTitnok (Rv Szilvia,
 * Szab Pter <pts@inf.bme.hu>, Szurdi Mikls, Weizengruber Attila).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or   
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package eNTitanok.kkkg;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.io.ObjectInputStream;
import java.io.Serializable;

/**
 * A jtk cscslistjnak (Hsi Halottaink, hall of fame, toplista, legjobbak
 * listja) adatszerkezett reprezentlja. A cscslistt (mint objektumot,
 * serialization-nel) ki lehet menteni lemezre s vissza is lehet tlteni.
 */
public class Csucslista implements Serializable {
  public static class CsucslistaBejegyzes implements Serializable {
    // `implements Comparable', de az nincs a JDK1.1-ben :-(
    // muszj `implements Serializable', nem elg Csucslistnak
    protected String nev;
    protected int pontszam;
    public CsucslistaBejegyzes(String nev, int pontszam) {
      this.nev=nev;
      this.pontszam=pontszam;
    }
    public int compareTo(Object masik) {
      CsucslistaBejegyzes masikcs=(CsucslistaBejegyzes)masik;
      return this.pontszam==masikcs.pontszam ? 0 :
             this.pontszam> masikcs.pontszam ? 1 : -1;
    }
    public String toString() { return nev+": "+pontszam; }
    public void hozzafuz(char c) { nev+=c; }
  }
  protected String filenev;
  public static final int MERET=9, NOVEKMENY=100;
  private CsucslistaBejegyzes bejegyzesek[];
  protected void alapErtek() {
    /* alaprtkekkel feltlttt :Cscslistt hoz ltre */
    bejegyzesek=new CsucslistaBejegyzes[MERET];
    for (int i=0;i<bejegyzesek.length;i++) bejegyzesek[i]=new CsucslistaBejegyzes("eNTitnok", (bejegyzesek.length-1-i)*NOVEKMENY);
  }
  public Csucslista setFilenev(String filenev) {
    this.filenev=filenev;
    return this;
  }
  public Csucslista() { alapErtek(); }
  public static Csucslista betolt(String filenev) {
    if (filenev==null) return new Csucslista();
    Csucslista ures=new Csucslista().setFilenev(filenev);
    try {
      FileInputStream f;
      ObjectInputStream o;
      try { f=new FileInputStream(filenev); }
      catch (IOException e) { return ures; }
      // catch (FileNotFoundException e) { return new Csucslista(); }
      try { o=new ObjectInputStream(f); }
      catch (java.io.StreamCorruptedException e) { return ures; } 
      catch (IOException e) { return ures; } 
      try { return ((Csucslista)o.readObject()).setFilenev(filenev); }
      catch (IOException e) { return ures; }
      catch (ClassNotFoundException e) { return ures; }
    } catch (Exception e) {
      // hlye bngszk, appletviewerek ezerfle szabvnytalan kivtelt
      // kpesek kldeni.
      return ures;
    }
  }
  public void kiment() {
    if (filenev==null) return;
    FileOutputStream f;
    try {
      f=new FileOutputStream(filenev);
      ObjectOutputStream o=new ObjectOutputStream(f);
      o.writeObject(this);
    }
    //catch (IOException e) {}
    catch (Exception e) {}
  }
  /**
   * Az utoljra felvett bejegyzs indexe, vagy `-1', ha nem volt ilyen.
   */
  protected int utoljara_felvett=-1;
  public int felvesz(CsucslistaBejegyzes csb) {
    /* felveszi a bejegyzst a cscslistba, ha elg nagy a pontszm.
     * Visszadja az indexet, ahov felvtelre kerlt, vagy -1.
     */
    int i=0;
    while (i<bejegyzesek.length && csb.compareTo(bejegyzesek[i])<0) i++;
    if (i==bejegyzesek.length) return utoljara_felvett=-1;
    System.arraycopy(bejegyzesek, i, bejegyzesek, i+1, bejegyzesek.length-i-1);
    bejegyzesek[i]=csb;
    return utoljara_felvett=i;
  }
  public int felvesz(int pontszam) {
    return felvesz(new CsucslistaBejegyzes("", pontszam));
  }
  
  /**
   * Az utoljra felvett bejegyzs nevhez fz egy karaktert.
   */
  public void hozzafuz(char c) {
    if (utoljara_felvett!=-1) bejegyzesek[utoljara_felvett].hozzafuz(c);
  }
  
  /**
   * Az utoljra felvett bejegyzs nevt zrja le, s visszaadja a bejegyzst
   * String debug info formjban. (null is lehet)
   */
  public String lezar() {
   int i=utoljara_felvett;
   if (i==-1) return null;
   utoljara_felvett=-1;
   return bejegyzesek[i].toString();
  }
  
  public void kiir() {
    for (int i=0;i<bejegyzesek.length;i++) System.out.println(bejegyzesek[i].toString());
    System.out.println("");
  }

  public CsucslistaBejegyzes elementAt(int i) { return bejegyzesek[i]; }
  public String nevAt(int i) { return bejegyzesek[i].nev; }
  public int pontszamAt(int i) { return bejegyzesek[i].pontszam; }
  public int length() { return bejegyzesek.length; }
} /// class Csucslista
