/* 
 * Sor.java
 * Object-ekbl ll, tetszleges hosszra nv FIFO megvalsts
 * by pts@fazekas.hu at Sat Apr  7 12:10:32 CEST 2001
 * tovbbi doksi: a README-ben s e file legkls class-nak fejkommentjben
 *
 * Kincskeres Kisgmbc (C) Early May 2001 by eNTitnok (Rv Szilvia,
 * Szab Pter <pts@inf.bme.hu>, Szurdi Mikls, Weizengruber Attila).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or   
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package eNTitanok.util;

/**
 * Egy generikus (Object-ekbl ll), tetszleges hosszra nv sor (queue,
 * FIFO) megvalsts. A sor elejrl lehet olvasni s a vghez lehet
 * hozzfzni. Thread safe (nem teszteltk).
 */
public class Sor {
  /**
   * A minimlis kapacits.
   */
  public static final int MIN_HOSSZ=16;
  protected Object t[];
  /**
   * Az elsknt belerakott, kiolvassra vr elem indexe.
   */
  protected int elso;
  /**
   * Az index, ahov a kvetkez beszrs trtnhet.
   */
  protected int uj;
  /**
   * Egy res sort hoz ltre, 16 kezdkapacitssal.
   */
  public Sor() { this(16); }
  /**
   * Egy adott kezdkapacits, res sort hoz ltre.
   */
  public Sor(int kezdomeret) {
    t=new Object[Math.min(kezdomeret,MIN_HOSSZ)];
    elso=uj=0;
  }

  public Sor klonoz() {
    Sor masik=new Sor();
    for (int i=elso;i<uj;i++) masik.vegehezFuz(t[i]);
    return masik;
  }
  
  /**
   * @return Pontosan akkor <code>true</code>, ha a sor res, azaz nincs benne
   * senki.
   */
  public synchronized boolean isUres() { return elso==uj; }
  /**
   * @return a sorbanll elemek szma.
   */
  public synchronized int length() { return uj-elso; }
  /**
   * Ha a sor legfeljebb a kapacits negyedt tlti ki, sszenyomja a felre.
   */
  protected synchronized void osszenyom() {
    if ((uj-elso)<=t.length/4 && t.length>=MIN_HOSSZ*2) {
      Object ujt[]=new Object[t.length/2];
      System.arraycopy(t, elso, ujt, 0, uj-elso);
      t=ujt; uj-=elso; elso=0;
    }
  }
  /**
   * Helyet csinl mg <code>mennyinek</code> j elemnek.
   */
  protected synchronized void helyetCsinal(int mennyinek) {
    if (t.length-uj<mennyinek) {
      int ujhossz=t.length;
      while (ujhossz-(uj-elso)<mennyinek) ujhossz*=2;
      Object ujt[]=ujhossz==t.length?t:new Object[ujhossz];
      System.arraycopy(t, elso, ujt, 0, uj-elso);
      t=ujt; uj-=elso; elso=0;
    }
  } /// helyetCsinal()
  /**
   * @return a sor elejn ll elem
   */
  public synchronized Object elso() { return uj==elso?null:t[elso]; }
  /**
   * Trli a sor elejn ll elemet.
   * @return az pp trlt elem
   */
  public synchronized Object elsoTorol() {
    if (uj==elso) return null;
    Object o=t[elso++];
    osszenyom();
    return o;
  }
  /**
   * @return a sor vgn ll elem
   */
  public synchronized Object utolso() { return uj==elso?null:t[uj-1]; }
  /**
   * Trli a sor vgn ll elemet. Br ezt a klasszikus <I>sor</I>
   * adatszerkezettl nem kveteljk meg, itt megvalstottuk.
   * @return az pp trlt elem
   */
  public synchronized Object utolsoTorol() {
    if (uj==elso) return null;
    Object o=t[--uj];
    osszenyom();
    return o;
  }
  /**
   * Paramteret a sor vghez fzi j elemknt.
   */
  public synchronized void vegehezFuz(Object ujelem) {
    helyetCsinal(1);
    t[uj++]=ujelem;
  }
  /**
   * Paramteret a sor vghez fzi j elemknt. Hibt okoz, ha `ujelem==null'.
   */
  public synchronized void vegehezFuzNemNull(Object ujelem) {
    if (ujelem==null) throw new NullPointerException();
    helyetCsinal(1);
    t[uj++]=ujelem;
  }
} /// class Sor
