/* 
 * Veletlen.java 
 * klnbz eloszls vletlenszmok generlja, determinisztikuss tehet
 * by pts@fazekas.hu at Sat Apr  7 12:10:32 CEST 2001, mayor update at Sun Apr 22 13:00:49 CEST 2001
 * tovbbi doksi: a README-ben s e file legkls class-nak fejkommentjben
 *
 * Kincskeres Kisgmbc (C) Early May 2001 by eNTitnok (Rv Szilvia,
 * Szab Pter <pts@inf.bme.hu>, Szurdi Mikls, Weizengruber Attila).
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or   
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package eNTitanok.util;
import eNTitanok.util.Sor;
import java.util.Random;

/**
 * Egy olyan (pszeudo)vletlenszm-genertor, amely determinisztikuss
 * tehet, azaz megmondhat neki, hogy a kvetkez nhny ,,vletlenszm''
 * mi legyen. Ezeket a fix szmokat az n. determinizl sorbl veszi. Egyetlen
 * genertorhoz egyetlen determinizl sor tartozik, ami kezdetben res.
 *
 * <P>A generlt vletlenszmok 0 s N-1 kz es int-ek, ahol N a
 * kovetkezo() hvs paramtere.
 * @see java.util.Random
 */
public class Veletlen {
  /**
   * A determinizl sor.
   */
  protected Sor kovetkezo;
  /**
   * Pontosan akkor `true', ha a generlt szmokat kizrlag a determinizl
   * sorbl szabad venni.
   */
  protected boolean csakDet;
  /**
   * Akkor vltdik ki, ha vletlenszmot kell generlni, `csakDet' igaz,
   * tovbb `kovetkezo' res.
   */
  public static class CsakDetException extends Error {};
  /**
   * Akkor vltdik ki, ha a `kovetkezo'-ben szerepl szm nem esik a megadott
   * hatrok kz.
   */
  public static class DetHatarException extends Error {
    public DetHatarException(String s) { super(s); }
    public DetHatarException(int also, int x, int felso) { super(also+" <= "+x+" < "+felso+" kene"); }
    public DetHatarException() { super(); }
  };
  /**
   * A hasznlt pszeudovletlenszm-genertor.
   */
  protected Random random;
  /**
   * Egy j vletlenszm-genertort hoz ltre, amely a Java alaprtelmezs
   * szerinti vletlenszm-genertort hasznlja.
   * @see java.util.Random
   */
  public Veletlen() { this(new Random(), false); }
  /**
   * Egy j vletlenszm-genertort hoz ltre, amely a megadott
   * vletlenszm-genertort hasznlja, s a megadott szm determinizl
   * sort kezel.
   */
  public Veletlen(Random random, boolean csakDet) {
    this.random=random==null?new Random():random;
    this.csakDet=csakDet;
    kovetkezo=new Sor();
  }
  /**
   * j rtket fz a determinizl sor vghez.
   */
  public void sorba(int ujertek) {
    kovetkezo.vegehezFuz(new Integer(ujertek));
    osszeg+=ujertek;
  }
  /**
   * <U>aasdasd</U>-nl nagyobb-egyenl, de <I>b</I>-nl kisebb vletlenszmot vesz
   * ki `kovetkezo' elejrl. Kivltja a megfelel kivteleket. `null'-t ad
   * vissza, ha `kovetkezo' res volt.
   */
  protected synchronized Integer csakDetet(int a, int b) {
    // System.out.println(kovetkezo.length()+"L"); // DBG
    Integer i=((Integer)kovetkezo.elsoTorol());
    if (i==null) {
      if (csakDet) throw new CsakDetException();
    } else {
      int iv=i.intValue();
      if (iv<a || iv>=b) throw new DetHatarException(a,iv,b);
    }
    return i;
  }
  /**
   * 0 s <I>n-1</I> kztti, egyenletes eloszls, egsz rtk
   * ,,vletlenszmot'' generl (a hatrokat is belertve),
   * az rtket a megadott determinisztikus sorbl vve, ha az nem res.
   * @param n ennl hatrozottan kisebb lesz a generlt szm. Ne legyen
   *        nagyobb Integer.MAX_VALUE/2-nl!
   */
  public int egyenletes(int n) {
    Integer i=csakDetet(0, n);
    if (i!=null) return i.intValue();
    int r=random.nextInt();
    return (r<0?n-r:r)%n;
  }
  /**
   * <I>a</I> s <I>b-1</I> kztti, egyenletes eloszls, egsz rtk
   * ,,vletlenszmot'' generl (a hatrokat is belertve),
   * az rtket a megadott determinisztikus sorbl vve, ha az nem res.
   * @param n ennl hatrozottan kisebb lesz a generlt szm. Ne legyen
   *        nagyobb Integer.MAX_VALUE/2-nl!
   */
  public int egyenletes(int a, int b) {
    Integer i=csakDetet(a, b);
    if (i!=null) return i.intValue();
    int r=random.nextInt();
    return (r<0?(b-a)-r:r)%(b-a)+a;
  }
  /**
   * Poisson-eloszls (<I>lambda</I> paramterrel), egsz rtk 
   * ,,vletlenszmot'' generl. Az tlagos futsid <I>lambda</I>-val
   * arnyos. Vrhat rtk: <I>lambda</I>. Szrs: <I>sqrt(lambda)</I>.
   * Csak felso-nl kisebb (nemnegatv) szmot ad vissza.
   * Minimum: 0. Maximum: amennyi belefr.
   *   <P>See Knuth, ACP, Section 3.4.1 Paragraph F.
   */
  public int poisson(double lambda, int felso) {
    if (lambda<=0.0) throw new ArithmeticException();
    Integer i=csakDetet(0, Integer.MAX_VALUE);
    if (i!=null) return i.intValue();
    double hatar=Math.exp(-lambda);
    double szorzat=1;
    int n;
    do {
      for (n=-1;szorzat>hatar;n++) szorzat*=random.nextDouble();
    } while (n<0 || n>=felso);
    return n;
  }
  /**
   * Csakgy, mint ktparamteres vltozata, de a fels hatr
   * <code>Integer.MAX_VALUE</code>.
   */
  public int poisson(double lambda) {
    return poisson(lambda, Integer.MAX_VALUE);
  }
  /**
   * Exponencilis eloszls (<I>lambda</I> paramterrel) ,,vletlenszm''
   * kerektett (<code>Math.round()</code>), egsz rtkt generlja.
   * Csak felso-nl kisebb (nemnegatv) szmot ad vissza.
   * A futsid O(1). Vrhat rtk: <I>1/lambda</I>.
   * Szrs: <I>1/lambda</I>. Minimum: 0. Maximum: amennyi belefr.
   * @see java.lang.Math#round
   */
  public int exponencialis(double lambda, int felso) {
    if (lambda<=0.0 || felso<1) throw new ArithmeticException();
    Integer i=csakDetet(0, felso);
    if (i!=null) return i.intValue();
    int n;
    do {
      n=Math.round((float)(-Math.log(random.nextDouble())/lambda));
    } while (n<0 || n>=felso);
    return n;
  }
  /**
   * Csakgy, mint ktparamteres vltozata, de a fels hatr
   * <code>Integer.MAX_VALUE</code>.
   */
  public int exponencialis(double lambda) {
    return exponencialis(lambda, Integer.MAX_VALUE);
  }
  /**
   * Normlis (Gauss-)eloszls (<I>m</I> vrhat rtk, <I>sigma</I>
   * szrs) ,,vletlenszm''
   * kerektett (<code>Math.round()</code>), egsz rtkt generlja.
   * Csak `felso'-nl kisebb szmot ad vissza.
   * Csak `also'-nl nagyobb-egyenlo szmot ad vissza.
   * A futsid JRE-fgg.
   * @see java.lang.Math#round
   */
  public int normalis(double m, double sigma, int also, int felso) {
    if (sigma<=0.0 || felso<1) throw new ArithmeticException();
    Integer i=csakDetet(also, felso);
    if (i!=null) return i.intValue();
    int n;
    do {
      // n=Math.round((float)((random.nextGaussian()-m)/sigma));
      n=Math.round((float)(random.nextGaussian()*sigma+m));
      // System.err.println(n); // DEBUG
    } while (n<also || n>=felso);
    return n;
  }
  /**
   * Csakgy, mint hromparamteres vltozata, de a fels hatr
   * <code>Integer.MAX_VALUE</code>.
   */
  public int normalis(double m, double sigma) {
    return normalis(m, sigma, Integer.MIN_VALUE, Integer.MAX_VALUE);
  }
  /**
   * Csakgy, mint ktramteres vltozata, de standard normlis eloszls
   * szerint generl (<I>m=0, sigma=1</I>).
   */
  public int normalis() {
    return normalis(0, 1, Integer.MIN_VALUE, Integer.MAX_VALUE);
  }
  /**
   * @return a hasznlt vletlenszm-genertor
   */
  public Random getRandom() { return random; }
  /**
   * @return az utolsnak elrt veletlenszam (0, ha nincs)
   */
  public int getUtolso() {
    if (kovetkezo.isUres()) return 0;
    return ((Integer)kovetkezo.utolso()).intValue();
  }
  
  /**
   * Az eddig sorba() helyezettek sszege.
   */
  private int osszeg=0;
  /**
   * j rtket fz a determinizl sor vghez, de kivonja belle az utols
   * elemet.
   */
  public void sorbaDelta(int ujertek) {
    // System.err.println(kovetkezo.length()+" uj="+ujertek+" ut="+osszeg); // DBG
    // System.err.println("EGESZ:"+kovetkezo.dump());
    kovetkezo.vegehezFuz(new Integer(ujertek-osszeg));
    osszeg=ujertek;
  }
  
  // Most a bels osztlyok kvetkeznek. Azrt nem kln .java file-ba raktam
  // ket, mert knyelmesebb egytt kezelni s ltni ket, ers kztk a
  // kohzi (no meg encapsulation).

  /**
   * Az t megvalst objektumok kpesek <code>int</code> tpus
   * vletlenszmokat generlni. Az intervallum s az eloszls tetszleges.
   */
  public static interface Generalo {
    /**
     * Generlja a kvetkez vletlenszmot.
     */
    public int general();
    /**
     * j elemet helyez el a vletlensorba.
     */
    public void sorba(int ujertek);
    /**
     * j rtket fz a determinizl sor vghez, de kivonja belle az utols
     * elemet.
     */
    public void sorbaDelta(int ujertek);
  }
  
  /**
   * <I>[a,b)</I>-n egyenletes eloszls szmokat generl.
   */
  public static class Egyenletes extends Veletlen implements Generalo {
    /**
     * Als hatr, belelrtend az intervallumba.
     */
    protected int a;
    /**
     * Fels hatr, nem tartozik az intervallumhoz.
     */
    protected int b;
    /**
     * @see Veletlen#Veletlen
     */
    public Egyenletes(Random random, boolean csakDet, int a, int b) {
      super(random, csakDet);
      this.a=a; this.b=b;
    }
    public Egyenletes(int a, int b) {
      super();
      this.a=a; this.b=b;
    }
    /**
     * tlltja a fels hatrt.
     */
    public void setB(int b) { this.b=b; }
    /**
     * Generlja a kvetkez vletlenszmot.
     */
    public int general() {
      return this.egyenletes(a, b);
    }
  } /// class Egyenletes
  
  /**
   * <I>[d,b+d)</I>-n <I>lambda</I> paramter, Poisson-eloszls szmoknl
   * $d$-vel nagyobb szmokat generl.
   */
  public static class Poisson extends Veletlen implements Generalo {
    /**
     * Als hatr, eltols, belelrtend az intervallumba.
     */
    protected int d;
    /**
     * Fels hatr (eltols nlkl), nem tartozik az intervallumhoz.
     */
    protected int b;
    /**
     * Az eloszls paramtere. (Egyben vrhat rtke s szrsngyzete.)
     */
    protected double lambda;
    /**
     * @see Veletlen#Veletlen
     */
    public Poisson(Random random, boolean csakDet, int b, int d, double lambda) {
      super(random, csakDet);
      this.b=b; this.d=d; this.lambda=lambda;
    }
    public Poisson(int b, int d) {
      super();
      this.b=b; this.d=d; this.lambda=lambda;
    }
    /**
     * Generlja a kvetkez vletlenszmot.
     */
    public int general() {
      return d+this.poisson(lambda, b);
    }
  } /// class Poisson

  /**
   * <I>[d,b+d)</I>-n <I>lambda</I> paramter, exponencialis eloszls
   * szmok egszre kerektettjeinl
   * $d$-vel nagyobb szmokat generl.
   */
  public static class Exponencialis extends Veletlen implements Generalo {
    /**
     * Als hatr, eltols, belelrtend az intervallumba.
     */
    protected int d;
    /**
     * Fels hatr (eltols nlkl), nem tartozik az intervallumhoz.
     */
    protected int b;
    /**
     * Az eloszls paramtere. (Egyben vrhat rtknek s szrsnak is
     * reciproka.)
     */
    protected double lambda;
    /**
     * @see Veletlen#Veletlen
     */
    public Exponencialis(Random random, boolean csakDet, int b, int d, double lambda) {
      super(random, csakDet);
      this.b=b; this.d=d; this.lambda=lambda;
    }
    public Exponencialis(int b, int d) {
      super();
      this.b=b; this.d=d; this.lambda=lambda;
    }
    /**
     * Generlja a kvetkez vletlenszmot.
     */
    public int general() {
      return d+this.exponencialis(lambda, b);
    }
  } /// class Exponencialis

  /**
   * <I>[a,b)</I>-n <I>m</I> vrhat rtk, <I>sigma</I> szrs,
   * normlis eloszls szmok egszre kerektettjeinl
   * $d$-vel nagyobb szmokat generl.
   */
  public static class Normalis extends Veletlen implements Generalo {
    /**
     * Als hatr, belelrtend az intervallumba.
     */
    protected int a;
    /**
     * Fels hatr, nem tartozik az intervallumhoz.
     */
    protected int b;
    /**
     * Az eloszls egyik paramtere, egyben vrhat rtke.
     */
    protected double m;
    /**
     * Az eloszls msik paramtere, egyben szrsa.
     */
    protected double sigma;
    /**
     * @see Veletlen#Veletlen
     */
    public Normalis(Random random, boolean csakDet, int a, int b, double m, double sigma) {
      super(random, csakDet);
      this.b=b; this.a=a; this.m=m; this.sigma=sigma;
    }
    public Normalis(int a, int b) {
      super();
      this.b=b; this.a=a; this.m=m; this.sigma=sigma;
    }
    /**
     * Generlja a kvetkez vletlenszmot.
     */
    public int general() {
      // System.err.println("DDD"); // DEBUG
      int i=this.normalis(m, sigma, a, b);
      // System.err.println("Gen."); // DEBUG
      return i;
    }
  } /// class Normalis
  /**
   * A megadott diszkrt, vges eloszls, nemnegatv rtk szmokat
   * generl.
   */
  public static class Veges extends Veletlen implements Generalo {
    /**
     * `eloszlas' elemeinek sszege. Bele kell, hogy frjen az int-be.
     */
    protected int osszeg;
    /**
     * Az eloszls. Nemnegatv gyakorisgrtkek, melyek sszege pozitv.
     * Az `i' kimenetel valsznsge: <I>eloszlas[i]/osszeg</I>.
     */
    protected int[] eloszlas;
    /**
     * Segdfggvny pl. `osszeg' kiszmtsra.
     */
    private synchronized void osszegez(int eloszlas[]) {
      if (eloszlas==null || eloszlas.length==0) throw new ArithmeticException();
      this.eloszlas=eloszlas;
      osszeg=0;
      for (int i=eloszlas.length-1;i>=0;i--) osszeg+=eloszlas[i];
      if (osszeg<1) throw new ArithmeticException();
    }
    /**
     * @see Veletlen#Veletlen
     */
    public Veges(Random random, boolean csakDet, int eloszlas[]) {
      super(random, csakDet);
      this.eloszlas=eloszlas; osszegez(eloszlas);
    }
    public Veges(int eloszlas[]) {
      super();
      this.eloszlas=eloszlas; osszegez(eloszlas);
    }
    /**
     * Generlja a kvetkez vletlenszmot.
     */
    public int general() {
      Integer ii=this.csakDetet(0, eloszlas.length);
      if (ii!=null) return ii.intValue();
      int n=this.egyenletes(osszeg);
      int i;
      for (i=eloszlas.length-1;i>=0;i--)
        if (0>(n-=eloszlas[i])) return i;
      throw new ArithmeticException(); /* notreached */
    }
  } /// class Veges
} /// class Veletlen
